--functions for the overview button to show the user an overview of all automations of all tracks in the song

--dialog globals
vb2 = nil 
my_dialog_2 = nil

require "ov_bypass_functions"

--function gets info on all the automations in the song (envelope automations)
-----------------------------------------------------
local function get_all_automated_parameters_in_song()
-----------------------------------------------------
  
  local song = renoise.song()
  local track_idx = song.selected_track_index
  
  --store names of devices and their objects
  --table to hold info for all tracks 
  local trk_tab = {}
  
  
  --Method:  
  --Loop all tracks
  --go through each pattern_track object for that track getting the automation[] table --renoise.song():pattern(pats):track(trk).automation
  --As we go build dev_table[i].automated_params = {} (and other relevant params)
  --by checking ":track(trk).automation" we can get the parent device pointed to, so no need to loop every parameter in each device to see if it is automated 
  --(some plugin devices can have large amounts of parameters)
  
  --loop tracks
  for trk = 1,#song.tracks do
     
    local track = song:track(trk)
    --trk_tab is the root table for all the tracks in the song.  We add the dev_table for each track once it is populated by looping
    trk_tab[trk] = {}
   
  
    --GET DEVICES IN TRACK
    ------------------
    local dev_table = {} --base {} holds the indexes (empty table,all indexes [x] return nil)
    --loop to populate dev tables
    for i = 1,#track.devices do
      --[i] will be holding another table
      dev_table[i] = {}
      --add new key and values
      dev_table[i].name = track.devices[i].short_name 
      dev_table[i].obj = track.devices[i]
      dev_table[i].index = i
      dev_table[i].automated_params = {} --table for automated parameters, this will be filled in by loops below
      dev_table[i].track_name = song:track(trk).name
      dev_table[i].track_index = trk
    end
    
    --reset flag at track loop level this will be added as a property "trk_tab[trk].has_automation" true/false
    local has_automation = false
    --loop track devices (to populate dev_table[i].automated_params{})
    ---------------------
    for dev = 1, #dev_table do
    
      --loop patterns (_tracks) to inspect all pattern_track automations
      ---------------------
      local cur_dev = dev_table[dev]
      for pats = 1,#song.patterns do
        
        local pat_tr = song:pattern(pats):track(trk) 
        local pat_autos = pat_tr.automation
        
         --automations present in pattern? --automation[] per pattern
         -------------------------------
         if #pat_autos > 0 then
           
           --if so loop all automations for current pattern track
           for i = 1,#pat_autos do
             --check if auto points to current device
             if rawequal(pat_autos[i].dest_device,cur_dev.obj) == true then
               
               --loop current  populate our cur_dev.automated_params {}  table
               local already_present = false
               for param = 1,#cur_dev.automated_params do
                 --check against parameter objects for the current pattern automations
                 if rawequal(pat_autos[i].dest_parameter,cur_dev.automated_params[param]) == true then
                   --match found so flag and break (we don't need to add this to the table,
                   -- it was added from a previous pattern_track automations[] table)
                   already_present = true
                   break
                 end
               end 
               --param wasnt added already so we add it now (#pat_autos loop adding all un-added automated-parameters )
               if already_present == false then
                 table.insert(cur_dev.automated_params, pat_autos[i].dest_parameter)
                 --automated parameter found in this track so make sure flagged to add as property
                 has_automation = true               
               end
             end
           end
         end
       end
     end
     --add dev_table for this track into main track table
     trk_tab[trk].dev_table = dev_table
     trk_tab[trk].has_automation = has_automation
   end --for track
 
  --return
  return trk_tab     
end

--function takes a table built in the function above: get_all_automated_parameters_in_song()
--Turns the info into an items table used for the popup list and its notifier.  This popup is used ta access any of all automations in the song
----------------------------------------
function get_popup_items(passed_trk_tab)
----------------------------------------
   
   local trk_tab = passed_trk_tab
   local items = {}
   local song = renoise.song()
 --  items.popup_items = {} --for constructor
 --  items.popup_items.track_name = nil
 --  items.popup_selections = {} --for notifier
 --  items.popup_selections.track_index = nil
 --  items.popup_selections.device_index = nil
 --  items.popup_selections.parameter_index = nil
   
  
  -- rprint(trk_tab)
   
   --loop all tracks
   local items_inc = 0
   local track_inc = 0
   
   --first entry 
   items_inc = items_inc+ 1
   items[items_inc] = {}
   items[items_inc].entry_name = "---"
   
   while 1 do
     
     --increment track count
     track_inc = track_inc + 1
     
     --Add Track Name To items If It Is Automated 
     ------------------------------
     if trk_tab[track_inc].has_automation == true then
       --Track device always present so we can get [1] here
       local dev_table = trk_tab[track_inc].dev_table[1]
       --incremant items table/ incrementor
       items_inc = items_inc + 1
       items[items_inc] = {}
       --add the track (with automation) as an entry for the popup
       items[items_inc].entry_name = "Track: "..dev_table.track_name..""
      -- items[items_inc].entry_name = tostring(dev_table.track_index)..": ".."("..dev_table.track_name..")"
       items[items_inc].entry_track_index = track_inc --add track index for popup notifier
     end
     
    
     local param_count = 1
     --loop each device in track
     for devs = 1,#trk_tab[track_inc].dev_table do
       --get the devs one by one
       local dev_table = trk_tab[track_inc].dev_table[devs]
       
       --rprint(dev_table)
       
       --if there are automations
       if #dev_table.automated_params > 0 then
         
         --add automated params for this device
         for param = 1,#dev_table.automated_params do
           items_inc = items_inc+ 1
           items[items_inc] = {}
           --limit device name
           local dev_name = dev_table.name
           if #dev_name > 12 then
             dev_name = string.sub(dev_name,1,12)..".."
           end
           
           --print(dev_table.automated_params[1].name)
           items[items_inc].entry_name = "     "..""..tostring(param_count)..". "..dev_name.." -- "..dev_table.automated_params[param].name
           param_count = param_count + 1
           items[items_inc].entry_track_index = track_inc --add track index for popup notifier
           items[items_inc].parameter = dev_table.automated_params[param]
           items[items_inc].device = dev_table.obj
           items[items_inc].index = devs
         end
         
         
       end--if
     end --for devs
     
     --add a spacer entry if parameteters were added 
     if param_count > 1 then
       items_inc = items_inc+ 1
       items[items_inc] = {}
       items[items_inc].entry_name = ""
       items[items_inc].entry_track_index = track_inc--set index as current track 
      end
      --break loop after last track
     if track_inc == #song.tracks then
       break
     end
   end--while
      
   return items
end

-------------------------------------------
function get_bypassed_for_selected_device()
-------------------------------------------
  local song = renoise.song()
  local track = song.selected_track
  local pat_seq = song.sequencer.pattern_sequence
  local max_cols = renoise.InstrumentPhrase.MAX_NUMBER_OF_EFFECT_COLUMNS
  --loop all patterns available in sequencer (we don't need hidden/ unused patterns)
  for i = 1,#pat_seq do
   -- renoise.song().patterns[].tracks[].lines[].effect_columns[]
    local pat_idx = song.sequencer:pattern(i)
    local cur_pat = song:pattern(pat_idx)
    if cur_pat.is_empty == false then
      --loop lines in pattern
      for lines = 1, #cur_pat.lines do
        local cur_line = cur_pat:line(lines)
        if cur_line.is_empty == false then
          --loop columns in line
          for fx_cols = 1,max_cols do
            --if acks[].lines[].effect_columns[].is_empty
            local cur_col = cur_line:effect_column(fx_cols)
            if cur_col.is_empty == false then
              if tonumber(cur_col.number_string) % 10 == 0 then
                --print("bypass found")
              end --amount_value
            end
          end
        end
      end
    end--pat empty 
  end
end


--show gui (called by a button on main gui)
-------------------------------
function show_overview_dialog()
-------------------------------

  

 
  local song = renoise.song()
  vb2 = renoise.ViewBuilder()
  
  ----------------------------------------------------------------
  --toggle dialog closed if open; makes keyboard shortcut a toggle
  ----------------------------------------------------------------
  if (my_dialog_2 and my_dialog_2.visible) then -- only allows one dialog instance
    my_dialog_2:close()
    --remove timer 
  --  if renoise.tool():has_timer(set_gui_values) then
     -- renoise.tool():remove_timer(set_gui_values)
   -- end
    vb2 = nil
    return
  end
  ---
  local all_autos_tab = get_all_automated_parameters_in_song()
  
 
  local items = get_popup_items(all_autos_tab)
  
  --get popup items from items table
  local popup_items = {}
  for i = 1,#items do
    popup_items[i] = items[i].entry_name
  end
  

  --function for popup notifier
  ----------------------------------------- 
  local function auto_popup_notifier(value) 
  -----------------------------------------
    --bypass main timer
    bypass_timer = true
  
    if value == 1 or items[value].entry_name == "" then -- blank entry
      return
    end
    --target track index
    local tar = items[value].entry_track_index
    local song = renoise.song()
    song.selected_track_index = tar
    --select device and parameter if a param was chosen in the popup
    if items[value].parameter ~= nil then
      --select device
      song.selected_device_index = items[value].index
      --update global so that main gui timer wont change parameters automatically again
      stored_selected_track_device =  song.selected_device
      --select parameter
      song.selected_parameter = items[value].parameter
      --select automation view
      --show the automation lane
      if (renoise.app().window.active_lower_frame == renoise.ApplicationWindow.LOWER_FRAME_TRACK_DSPS) then
        renoise.app().window.active_lower_frame = renoise.ApplicationWindow.LOWER_FRAME_TRACK_AUTOMATION
      end
    end
    bypass_timer = false
  end
  ---
  --[[
   --for bypass fx popup
   local bp_tab = get_items_for_bypass_popup(song.selected_track_index,song.selected_device_index-1)
   local bypassed_items = {}
   for i = 1,#bp_tab do
     bypassed_items[i] = bp_tab[i].item_string 
   end
   local first_entry = "n/a"
   if #bp_tab > 0 then
    -- first_entry = tostring("("..#bp_tab..")")
     first_entry = "---"
   end
   table.insert(bypassed_items, 1,first_entry)
   --]]
 
  --gui
  ------
  
  local dialog_content = vb2:column{ 
     margin = 8,
     vb2:text{
       text = "Selected Track:"
     },
     
     vb2:row{ --track name, black background
       margin = 2,
       style = "border",
       
       vb2:text{
         id = "tr name",
         text = song.selected_track.name,
         width = 150,
       },
      },
      
     vb2:text{
       text = "Automation In Song:",
     },

       vb:button{
         text = "Refresh",
         notifier = function()
         
                      --1) Update All Automations popup
                      ---------------------------------
                      --update items table
                      local song = renoise.song()
                      --get first table type
                      all_autos_tab = get_all_automated_parameters_in_song()
                      --pass to second function to get items table
                      items = get_popup_items(all_autos_tab)
                        --get popup items from items table
                      local popup_items = {}
                      for i = 1,#items do
                        popup_items[i] = items[i].entry_name
                      end
                      --repopulate popup
                      vb2.views["all autos popup"].items = popup_items
                      --loop and select the current automation in popup if it is selected in renoise
                      local param_found = false
                      for i = 1,#items do
                        if rawequal(items[i].parameter,song.selected_parameter) then
                          vb2.views["all autos popup"].value = i
                          param_found = true
                          break
                        end 
                      end
                      
                      if param_found == false then
                        vb2.views["all autos popup"].value = 1
                      end
                      --[[
                      --2) Update Bypasses for selected Device
                      ----------------------------------------
                       bp_tab = get_items_for_bypass_popup(song.selected_track_index,song.selected_device_index-1)
                       bypassed_items = {}
                       for i = 1,#bp_tab do
                         bypassed_items[i] = bp_tab[i].item_string 
                       end
                       if #bp_tab > 0 then
                         first_entry = tostring("("..#bp_tab..")")
                         first_entry = tostring("---")
                       end
                       table.insert(bypassed_items, 1,first_entry)
                       vb2.views["bypassed popup"].items = bypassed_items
                      --]]

                       status("Popup Refreshed")
                    end
       
       },
       
  

     vb2:popup{
       items = popup_items,-- track_names,
       id = "all autos popup",
       width = 250,
       notifier = function(value)
                    auto_popup_notifier(value)
                  end,
     },
     
     vb2:text{
       text = "-------------------------------------",
     },
     
    vb2:text{
       text = "Selected Device:",
     },
     vb2:row{
       margin = 2,
       style = "border",
       vb2:text{
         id = "selected dev name",
        
       }
       
     },
     
     vb2:text{
      text = "FX Col Bypasses For Selected Device:",
     },
     vb:row{
      vb2:button{
        text = "Prev",
        id = "prev button",
        notifier = function()
                     go_to_previous_col_fx_bypass_selected_device()
                   end
       
      },
       vb2:button{
         text = "Next",
         id = "next button",
         notifier = function()            
                       go_to_next_col_fx_bypass_selected_device()
                    end
      },
     },
     --[[
     vb2:popup{
       items = bypassed_items,
       width = 250,
       id = "bypassed popup",
       notifier = function(value)
                    if value == 1 then --"---"
                      return
                    end
                    --decrecrement value to account for "---" entry
                    value = value - 1
                    local song = renoise.song()
                    song.selected_sequence_index = bp_tab[value].sequencer_index
                    song.selected_line_index = bp_tab[value].line_index
                  end
     }
    --]] 
     --[[
     vb2:multiline_textfield{
       text ="", --get_textfield_text(all_autos_tab),
       width = 300,
       height = 300,
     
      }
      
      --]]
     }--column
  
 
  
  ------------------------------------------------------------------
  --key Handler passes key input to renoise when tool gui is focused
  ------------------------------------------------------------------
  local function my_keyhandler_func(dialog, key)
   --hack to always refocus the pattern editor so renoise responds to key input
   renoise.app().window.lock_keyboard_focus = not renoise.app().window.lock_keyboard_focus
   renoise.app().window.lock_keyboard_focus = not renoise.app().window.lock_keyboard_focus
   return key
  end
  ---
  --show and assign dialog
  local title = "Automation In Song"
  my_dialog_2 = renoise.app():show_custom_dialog(title,dialog_content,my_keyhandler_func)
  
  

  --add timer to fire once every 50ms
  if not renoise.tool():has_timer(timer_2) then
    renoise.tool():add_timer(timer_2,80)
  end

  
  
  --first run of timer
  timer_2()
  

end --of show_overview_dialog()
----

  




--timer function
------------------
function timer_2()
------------------  
  local song = renoise.song()
  
  --update track name
  vb2.views["tr name"].text = song.selected_track.name
  
  --select mixer device if no selection
  if song.selected_device == nil then
    song.selected_device_index = 1
  end
  if song.selected_device_index == 1 then
    vb2.views["prev button"].active = false
    vb2.views["next button"].active = false
  else
    vb2.views["prev button"].active = true
    vb2.views["next button"].active = true
  end
  
  vb2.views["selected dev name"].text = song.selected_device.short_name
  vb2.views["selected dev name"].width = 140
  
  
  
  --remove timer when GUI is closed
  if(my_dialog_2 == nil) or (my_dialog_2.visible == false) then
    if renoise.tool():has_timer(timer_2) then
      renoise.tool():remove_timer(timer_2)
    end
  end
end





